/*********************************************************************
 * Copyright 2011 Bartek thindil Jasicki
 *
 *  This file is part of Steam Fort.
 *
 *  Steam Fort is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Steam Fort is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Steam Fort.  If not, see <http://www.gnu.org/licenses/>.
*********************************************************************/
#include "utils.hpp"
#include "main.hpp"
#include <map>
#include <stdio.h>
#include "ui.hpp"
#include "item.hpp"
#include "map.hpp"
#include "adventure.hpp"

Container *itemsmenu = new Container(0, 0, 100, 75);

/**
 * Standard class constructor.
 * Create object and fill some default values.
 */
Item::Item()
{
  taken = false;
  stored = false;
  hidden = false;
  description = "";
  designated = false;
  data = NONE;
  damage = "";
  range = 1;
  use = 0;
}

/**
 * Add item to game
 * @param start Did we start game.
 * @return Index of new item.
 */
int Item::additem(bool start)
{
  int item = NONE;
  if (state == FORT)
    {
      if ((fortobjs.size() > 0) && (!start))
	{
	  for (int i = 0; i < (int)fortobjs.size(); i++)
	    {
	      if (((fortobjs[i].x == NONE) || (fortobjs[i].y == NONE)) && (fortobjs[i].hidden))
		{
		  fortobjs[i] = *this;
		  item = i;
		  break;
		}
	    }
	}
      if (item == NONE)
	{
	  item = fortobjs.size();
	  fortobjs.push_back(*this);
	}
    }
  else
    {
      if ((advobjs.size() > 0) && (!start))
	{
	  for (int i = 0; i < (int)advobjs.size(); i++)
	    {
	      if (((advobjs[i].x == NONE) || (advobjs[i].y == NONE)) && (advobjs[i].hidden))
		{
		  advobjs[i] = *this;
		  item = i;
		  break;
		}
	    }
	}
      if (item == NONE)
	{
	  item = advobjs.size();
	  advobjs.push_back(*this);
	}
    }
  return item;
}

/**
 * Show object on screen.
 * @param x1 X-axis position of object on map
 * @param y1 Y-axis position of object on map
 * @param iindex Index of item.
 * @param start Did we start game.
 */
void Item::setitem(int x1, int y1, int iindex, bool start)
{
  TCODRandom *generator = TCODRandom::getInstance();
  while (true)
    {
      if (valid_pos(x1, y1))
	{
	  x = x1;
	  y = y1;
	  if (iindex == NONE)
	    {
	      iindex = additem(start);
	    }
	  update_map_item(x, y, iindex);
	  break;
	}
      else
	{
	  x1 = generator->getInt(x1 - 5, x1 + 5);
	  if ((x1 < 0) || (x1 >= MAPWIDTH))
	    {
	      x1 = MAPWIDTH / 2;
	    }
	  y1 = generator->getInt(y1 - 5, y1 + 5);
	  if ((y1 < 0) || (y1 >= MAPHEIGHT))
	    {
	      y1 = MAPHEIGHT / 2;
	    }
	}
    };
}

/**
 * Use item (by mob/building).
 */
void Item::useitem()
{
  x = NONE;
  y = NONE;
}

/**
 * Delete item from game (not useable anymore)
 */
void Item::deleteitem()
{
  x = NONE;
  y = NONE;
  hidden = true;
}

/**
 * Build items user interface.
 */
void buildItemsUI()
{
  itemsmenu->setVisible(false);
}

/**
 * Show/hide information about items.
 * @param visible If true, show items, otherwise hide menu.
 */
void setVisibleItems(bool visible)
{
  if (visible)
    {
      //Get all items
      map<string, vector<int> > items;
      for (int i = 0; i < (int)fortobjs.size(); i++)
	{
	  if ((fortobjs[i].x > -1) && (fortobjs[i].y > -1) && (fortobjs[i].types.contains(OBJECT)))
	    {
	      if (items.find(fortobjs[i].name) == items.end())
		{
		  items[fortobjs[i].name].assign(2, 0);
		}
	      items[fortobjs[i].name][0] ++;
	      if (fortobjs[i].stored)
		{
		  items[fortobjs[i].name][1] ++;
		}
	    }
	}
      itemsmenu->clear();
      if (items.empty())
	{
	  itemsmenu->addWidget(new Label(35, 30, "No resources available"));
	}
      else
	{
	  itemsmenu->addWidget(new Label(5, 2, "Name"));
	  itemsmenu->addWidget(new Label(30, 2, "Amount"));
	  itemsmenu->addWidget(new Label(60, 2, "Stored"));
	  int line = 0;
	  char* c;
	  for (map<string, vector<int> >::iterator i = items.begin(); i != items.end(); i++)
	    {
	      itemsmenu->addWidget(new Label(5, 4 + line, (*i).first.c_str()));
	      c = new char[10];
	      sprintf(c, "%d", (*i).second[0]);
	      itemsmenu->addWidget(new Label(30, 4 + line, c));
	      c = new char[10];
	      sprintf(c, "%d", (*i).second[1]);
	      itemsmenu->addWidget(new Label(60, 4 + line, c));
	      line ++;
	    }
	}
    }
  itemsmenu->setVisible(visible);
  escinfo->setVisible(visible);
}

/**
 * Find proto object by name or type
 * @param name Name of object to find
 * @param types Types of object to find
 * @return Object index in protoobjs
 */
int find_proto_obj(std::string name, TCODList<int> *types)
{
  int obj = -1, hits;
  for (int i = 0; i < (int)protoobjs.size(); i++)
    {
      //Search by name
      if ((name.size() > 0) && (protoobjs[i].name.compare(name)))
	{
	  continue;
	}
      //Search by types
      if (!types->isEmpty())
	{
	  hits = 0;
	  for (int j = 0; j < types->size(); j++)
	    {
	      if (protoobjs[i].types.contains(types->get(j)))
		{
		  hits ++;
		}
	    }
	  if (hits != types->size())
	    {
		continue;
	    }
	}
      obj = i;
      break;
    }
  return obj;
}

/**
 * Find item by name, position or type.
 * @param name Name of item to find. If we don't want
 * search by name, put empty string here.
 * @param x X-axis position of item. Default is -1.
 * @param y Y-axis position of item. Default is -1.
 * @param types Types of item to find.
 * @param hidden Check for hidden items too.
 * @return Index of item in fortobjs vector. If not
 * found, return -1.
 */
int find_obj(std::string name, int x, int y, TCODList<int> *types, bool hidden)
{
  int obj = NONE, hits;
  //Search by position
  if ((x != NONE) && (y != NONE))
    {
      TCODList<int> tmplist;
      if (!hidden)
	{
	  tmplist = fortmap[y][x].items;
	}
      else
	{
	  tmplist = fortmap[y][x].hitems;
	}
      if (tmplist.size() == 0)
	{
	  return NONE;
	}
      int index;
      for (int i = 0; i < tmplist.size(); i++)
	{
	  index = tmplist.get(i);
	  if ((name.size() > 0) && (name.compare(fortobjs[index].name)))
	    {
	      continue;
	    }
	  if (!types->isEmpty())
	    {
	      hits = 0;
	      for (int j = 0; j < types->size(); j++)
		{
		  if (fortobjs[index].types.contains(types->get(j)))
		    {
		      hits ++;
		    }
		}
	      if (hits != types->size())
		{
		  continue;
		}
	    }
	  obj = index;
	  break;
	}
      return obj;
    }
  for (int i = 0; i < (int)fortobjs.size(); i++)
    {
      //Avoid used items
      if ((fortobjs[i].x == NONE) || (fortobjs[i].y == NONE))
	{
	  continue;
	}
      //Search by hidden
      if (fortobjs[i].hidden != hidden)
	{
	  continue;
	}
      //Search by name
      if ((name.size() > 0) && (name.compare(fortobjs[i].name)))
	{
	  continue;
	}
      //Search by types
      if (!types->isEmpty())
	{
	  hits = 0;
	  for (int j = 0; j < types->size(); j++)
	    {
	      if (fortobjs[i].types.contains(types->get(j)))
		{
		  hits ++;
		}
	    }
	  if (hits != types->size())
	    {
	      continue;
	    }
	}
      //Avoid taken item
      if (fortobjs[i].taken)
	{
	  continue;
	}
      obj = i;
      break;
    }
  return obj;
}

/**
 * Find nearest item from selected position
 * @param name Name of item to find.
 * @param x X-axis of start search position.
 * @param y Y-axis of start search position.
 * @param radius Size of radius to search.
 * @param types Types of item to find. Default empty.
 * @param stored Check for objects in storehouse? Default is true.
 * @param hidden Check for visible or not objects? Default is false (only visible).
 * @return Index of item in fortobjs vector.
 * If item not found, return -1.
 */
int find_near_obj(std::string name, int x, int y, int radius, TCODList<int> *types, bool stored, bool hidden)
{
  int obj = NONE, index, sum, oldsum = 1000, hits;
  TCODList<int> tmpitems;
  for (int y1 = y - (radius + 1); y1 < y + (radius + 1); y1++)
    {
      for (int x1 = x - (radius + 1); x1 < x + (radius + 1); x1++)
	{
	  if ((y1 < 0) || (x1 < 0) || (y1 >= MAPHEIGHT) || (x1 >= MAPWIDTH))
	    {
	      continue;
	    }
	  sum = abs(x - x1) + abs(y - y1);
	  if (sum > oldsum)
	    {
	      continue;
	    }
	  if (!hidden)
	    {
	      tmpitems = fortmap[y1][x1].items;
	    }
	  else
	    {
	      tmpitems = fortmap[y1][x1].hitems;
	    }
	  if (tmpitems.isEmpty())
	    {
	      continue;
	    }
	  for (int i = 0; i < tmpitems.size(); i++)
	    {
	      index = tmpitems.get(i);
	      if ((name.size() > 0) && (name.compare(fortobjs[index].name)))
		{
		  continue;
		}
	      if (!types->isEmpty())
		{
		  hits = 0;
		  for (int j = 0; j < types->size(); j++)
		    {
		      if (fortobjs[index].types.contains(types->get(j)))
			{
			  hits ++;
			}
		    }
		  if (hits != types->size())
		    {
		      continue;
		    }
		}
	      if (fortobjs[index].taken)
		{
		  continue;
		}
	      obj = index;
	      oldsum  = sum;
	      break;
	    }
	}
    }
  return obj;
}

/**
 * Find all prototypes with selected types.
 * @param types Item types to find.
 * @return List of indexes of objects.
 */
TCODList<int> find_all_proto_obj(TCODList<int> types)
{
  TCODList<int> results;
  for (int i = 0; i < (int)protoobjs.size(); i++)
    {
      for (int j = 0; j < types.size(); j++)
	{
	  if (protoobjs[i].types.contains(types.get(j)))
	    {
	      results.push(i);
	      break;
	    }
	}
    }
  return results;
}

/**
 * Find all objects with selected types.
 * @param types Item types to find.
 * @return List of indexes of objects.
 */
TCODList<int> find_all_obj(TCODList<int> types)
{
  TCODList<int> results;
  if (state == FORT)
    {
      for (int i = 0; i < (int)fortobjs.size(); i++)
	{
	  for (int j = 0; j < types.size(); j++)
	    {
	      if (((fortobjs[i].x == NONE) || (fortobjs[i].y == NONE)) && (fortobjs[i].hidden))
		{
		  continue;
		}
	      if (fortobjs[i].types.contains(types.get(j)))
		{
		  results.push(i);
		  break;
		}
	    }
	}
    }
  else
    {
      for (int i = 0; i < (int)advobjs.size(); i++)
	{
	  for (int j = 0; j < types.size(); j++)
	    {
	      if (((advobjs[i].x == NONE) || (advobjs[i].y == NONE)) && (advobjs[i].hidden))
		{
		  continue;
		}
	      if (advobjs[i].types.contains(types.get(j)))
		{
		  results.push(i);
		  break;
		}
	    }
	}
    }
  return results;
}
